@@ -164,6 +164,12 @@ class IPC:
                 await self.redis.publish(self.channel_address, resp)
         except asyncio.CancelledError:
             pass
+        except Exception as e:
+            on_error = getattr(self, 'on_error', None)
+            if on_error is not None:
+                await on_error(e, message)
+            else:
+                raise e from None
 
     async def ensure_channel(self) -> None:
         if self.channel is None:
@@ -187,7 +193,7 @@ class IPC:
                 required_identity = message.get('required_identity')
                 if op is None and sender != self.identity and nonce is not None and nonce in self.nonces:
                     future = self.nonces.get(nonce)
-                    if future:
+                    if future and not future.done():
                         future.set_result(data)
                     continue
 
